#! /usr/bin/env python

#############################################################
# Quick and dirty server to translate xmlrequests into key presses
#
# This code was written by Andrew McCargar but please don't hold it against him
# I release this code into the public domain.
# to read about this project, please visit www.outsideworld.org
#############################################################

import string,cgi,time,codecs,urllib
from os import curdir, sep, listdir, remove, path
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

import socket # For Utils
import struct # For Utils
import sys # For Utils
import commands

import mimetypes,random
from keycodes import keypairs

class Utils():
	def add_mimetypes(self):
		"""adds common mimetypes to python's built in mimetypes module"""
		filetypes ={'.html':'text/html; charset=utf-8','.js':'text/javascript','.css':'text/css','.gif':'image/gif','.png':'image/png','.jpg':'image/jpg','.ico':'images/x-icon'}
		for mimetype in filetypes.keys():
			mimetypes.add_type(filetypes[mimetype],mimetype)

	def get_ip_address(self, ifname):
		"""returns the server computer's ip"""
		if sys.platform[0:5] == 'linux':
			import fcntl
			s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
			return socket.inet_ntoa(fcntl.ioctl(
					s.fileno(),
					0x8915,	# SIOCGIFADDR
					struct.pack('256s', ifname[:15])
					)[20:24])
		else: #if running on windows or other Unix
			return socket.gethostbyname(socket.gethostname())

	def parse_path(self, path):
		"""splits the path at the first instance of '&' if any"""
		command = None
		if '?' in path:
			path, command = path.split('?')
			if '&' in command: #the spiderscript library puts extra characters at the end to make a unique request
				command, a = command.split('&') #in the future this should be done more inteligently
		return path, command

	def get_mimetype(self, path):
		mimetype = mimetypes.guess_type(path)[0]
		if not mimetype:
			mimetype = 'text/plain'
		return mimetype

	def send_key(self, code):
		"""tries to push out a key press using the xte command"""
		if (code == 'delete'):
			cmd = "xte 'key BackSpace'"
		elif (code == 'enter'):
			cmd = "xte 'key Return'"
		else:
			key = keypairs[code]
			cmd = "xte 'str " + key + "'";
		print "cmd: " + cmd
		try:
			dos = commands.getstatusoutput(cmd)
		except:
			print "could not press key\n"

class MyHandler(BaseHTTPRequestHandler):
	def do_SEND(self, file='', code=200, mimetype='text/plain'):
		"""sends a requested file to the client"""
		self.wfile.write("HTTP/1.1 %d\r\n" % code)
		self.wfile.write("Content-type: %s\r\n" % mimetype)
		self.wfile.write("\r\n")
		self.wfile.write(file)

	def do_GET(self):
		"""handles html GET requests"""
		path, command = Utils().parse_path(self.path)
		mimetype = Utils().get_mimetype(path)
		path = 'html'+path
		path = urllib.unquote(path)
		try:
			if path == 'html/':
				path += 'index.html'
				f = open(curdir + sep + path)
				self.do_SEND(f.read(), 200, 'text/html')
			else:
				if mimetype.startswith('text'): #Wndows needs to know if file is binary or not, Unix doesn't care
					f = open(curdir + sep + path)
				else:
					f = open(curdir + sep + path, 'rb')
				self.do_SEND(f.read(), 200, mimetype)
			f.close()
		except IOError:
			self.send_error(404,'File Not Found: %s' % path)

	def do_POST(self):
		"""handles http POST requests"""
		try:
			ctype, pdict = cgi.parse_header(self.headers.getheader('content-type'))
			if ctype == 'application/x-www-form-urlencoded':
				clen = self.headers.getheader('content-length')
				if clen:
					clen = int(clen)
				else:
					print 'POST ERROR: missing content-length'
					return
				input_body = self.rfile.read(clen)
				input_body = urllib.unquote(input_body)
				path, command = Utils().parse_path(self.path)
				if path == '/keyboard.cgi':
					if input_body:
#						print "input_body: " + input_body
						tmp, key = input_body.split("=")
						Utils().send_key(key)
						self.do_SEND("text",200)
						return
				self.send_error(500,'Internal Server Error, command not recognised')
			else:
				self.send_error(500,'Internal Server Error, POST type not recognised: %s' % ctype)
		except:
			pass

def main():
	try:
		PORT = "8000"
		Utils().add_mimetypes()
		server = HTTPServer(('', int(PORT)), MyHandler)
		#Change this to match your internet device
		serverip = Utils().get_ip_address("wlan0")
		print 'Server IP Address:', serverip
		print 'Port: ' + PORT
		print 'Point your browser to: http://'+str(serverip)+':'+PORT+'/'
		print 'started httpserver...'
		server.serve_forever()
	except KeyboardInterrupt:
		print '^C received, shutting down server'
		server.socket.close()

if __name__ == '__main__':
	main()